function [hat_x,info] = alg_rLSAPCi_inner(y,M,hat_x,nuks,Llow,Lhigh,indL,eL0,iterations,hat_upsilon)
% the main algorithm for Source term estimation of multi-nuclide atmospheric release of radiation from gamma dose rates

[p n] = size(M);

hat_L = eye(n);

en_x1 = (n/nuks)*(nuks-1)+1; % x_m

% initialization
for i=1:nuks
    inuk = [((i-1)*(n/nuks)+1):((i-1)*(n/nuks)+n/nuks)];
    
    ups_inuk = hat_upsilon(inuk);
    ups_inuk(ups_inuk == 0) = mean(hat_upsilon(inuk));
    hat_upsilon(inuk) = ups_inuk;

    x_inuk = hat_x(inuk);
    x_inuk(x_inuk == 0) = mean(hat_x(inuk));
    hat_x(inuk) = x_inuk;
end
hat_xxt = hat_x*hat_x';

%% sizes, initializations, and prior constants
hat_varsigma = 1e-5*indL;

alpha0 = 1e-10;
beta0 = 1e-10;
vartheta0 = 1e-10;
rho0 = 1e-10;

Dsi_lj = ones(n,n);
pomLtxxtL = zeros(n);
si_LLt = zeros(n);
pomLULt = 1*zeros(n);

info.omega = zeros(iterations,1);

%% main cycle
konec = 1;
iter = 0;
while konec
    iter = iter + 1;
    if mod(iter,10) == 0
        display(['rLSAPC-intervals: iteration ' num2str(iter) '/' num2str(iterations)])    
    end

    %% upsilon
    alpha = alpha0 + (1/2)*ones(n,1);
    for j = 1:(n-1)
        ind = find(indL(:,j)>0);
        pomLtxxtL(j,j) = hat_L([j; ind],j)'*hat_xxt([j; ind],[j; ind])*hat_L([j; ind],j) + trace(diag(Dsi_lj(ind,j))*hat_xxt(ind,ind));
    end
    pomLtxxtL(n,n) = hat_xxt(n,n);    
    beta = beta0 + (1/2)*diag(pomLtxxtL);
    hat_upsilon = alpha./beta;
    
    upspom = hat_upsilon(en_x1:end); % x_m
    for i = 1:15
        hat_upsilon((1+(i-1)*(n/nuks)):((n/nuks)+(i-1)*(n/nuks))) = upspom;
    end

 
    %% L
    pomLULt = zeros(n);

    for j = 1:(n) % columns
        ind = find(indL(:,j)>0); % active indexes

        si_lj = (hat_upsilon(j)*hat_xxt(ind,ind) + diag(hat_varsigma(ind,j)) )^(-1);
        mu_lj = si_lj*(-hat_upsilon(j)*(hat_xxt(ind,j) ) + diag(hat_varsigma(ind,j))*eL0(ind,j));
        if 1 % numerics
            si_lj = real(si_lj);
            mu_lj = real(mu_lj);
        end
        if j<en_x1 % truncations to specific intervals of L
            hat_lj = zeros(size(ind));
            hatll = zeros(size(ind));
            for i = 1:size(ind)
                [hat_lj(i) hatll(i)] = momtrun_tN(mu_lj(i),sqrt(abs(si_lj(i,i))),Llow(ind(i),j),Lhigh(ind(i),j));
            end
            var_lj = hatll - hat_lj.^2;
            hat_ljljt = hat_lj*hat_lj' + diag(var_lj);

        else % no interval for x_m
            hat_lj = mu_lj;
            hat_ljljt = mu_lj*mu_lj' + si_lj;
            hat_ljtlj = mu_lj'*mu_lj + sum(diag(si_lj));
        end

        hat_L(ind,j) = hat_lj;

        Dsi_lj(ind,j) = diag(si_lj);

        if j<en_x1 % for all except x_m
            zeta0 = 1e-10;
            eta0 = 1e-10;
            zeta = zeta0 + (1/2);
            eta = eta0 + (1/2)*diag(hat_ljljt) - diag(eL0(ind,j)*hat_lj') + (1/2)*diag(eL0(ind,j)*eL0(ind,j)');
            hat_varsigma(ind,j) = zeta./eta; 
        else % for x_m
            zeta0 = 1e-2; 
            eta0 = 1e-2;
            zeta = zeta0 + (1/2);
            eta = eta0 + (1/2)*diag(hat_ljljt) - diag(eL0(ind,j)*hat_lj') + (1/2)*diag(eL0(ind,j)*eL0(ind,j)');
            hat_varsigma(ind,j) = zeta./eta; 
        end

        si_LLt(ind,ind) = si_lj;
        pomLULt(ind,ind) = pomLULt(ind,ind) + si_lj*hat_upsilon(j); % pomocna matice pro x
    end
    hat_LLt = hat_L*hat_L' + si_LLt;
    
    %% omega
    vartheta = vartheta0 + p/2;
    rho = rho0 + (1/2)*trace(hat_xxt*M'*M) - y'*M*hat_x + (1/2)*y'*y;
    hat_omega = vartheta/rho;
    
    info.omega(iter) = hat_omega; 
    
    %% x
    si_x = (hat_omega*M'*M + hat_L*diag(hat_upsilon)*hat_L' + pomLULt )^(-1);
    mu_x = si_x*(hat_omega*M'*y);
    if 1 % numerics
        si_x = real(si_x);
        mu_x = real(mu_x);
    end
    
    [hat_x hat_xx] = momtrun_low(mu_x,sqrt(abs(diag( si_x ))),0);
    var_x = hat_xx - hat_x.^2;
    hat_xxt = hat_x*hat_x' + diag(var_x);
    if 1
        old_dSi = diag(si_x);
        new_dSi = var_x;
        nasobic = diag(sqrt(new_dSi./old_dSi));
        new_si_x = nasobic*si_x*nasobic;
        hat_xxt = hat_x*hat_x' + new_si_x;
    end 

%     keyboard
    %%
    if iter == iterations
        konec = 0;
    end
    
end

%% report results
info.L = hat_L;
info.L0 = eL0;
info.indL = indL;
info.Llow = Llow;
info.Lhigh = Lhigh;
info.varsigma = hat_varsigma;
info.ups = hat_upsilon;
info.mux = mu_x;
info.six = si_x;

end
