% example run of Multi-nuclide source term estimation algorithm
% based on paper submitted to QJRMS by O. Tichy, V Smidl, R. Hofman, and Nikolaos Evangeliou 
% contact: otichy@utia.cas.cz

clear all;
warning('off','all')

%% load data
load ./data/ratios_data.mat
% loaded y and M
[p,n] = size(M);

%% data scaling
coef = max(abs(y));
y = y./coef;
M = M./coef;

%% run main script
window_size = 14;
[hat_x,info] = alg_rLSAPC_intervals(y,M,window_size); 

%% print
load ./data/xt.mat
    % reordering
    ind = [];
    for i = 1:16
        ind = [ind i:16:n]; 
    end
    x_true = xt(ind);

% plot results
fig = figure;
set(fig, 'Position', [1500, 500, 1000, 500]);
rows = 4;
cols = 4;
nuks = 16;
nnuk = 14;
load ./data/nuc_names.mat % --> nuclides
for i = 1:nuks%(rows*cols)
    ind = (1+(i-1)*nnuk):(nnuk+(i-1)*nnuk);
        dsix = info.dsix;
        a1 = hat_x(ind) + 3*dsix(ind).^(1/2);
        a2 = hat_x(ind) - 3*dsix(ind).^(1/2);
        meze = [a2 (a1-a2)];
    subplot(rows,cols,i)
    h = area(meze);
    set(h(1),'FaceColor','white');
    set(h(2),'FaceColor',[189/255 255/255 182/255]);
    hold on;
    p1=plot(hat_x(ind),'LineWidth',2);
    hold on;
    p2=plot(x_true(ind),'red--','LineWidth',2);
    hold off;
    title(nuclides(i))
    if i >= 13
        xlabel('time (h)')
    end
    if mod(i,4)==1
        ylabel({'Release rate';'(Bq/s)'})
    end
    xlim([1 14])
    ylim([0 1.5*max([hat_x(ind)' x_true(ind)'])])
end






